from typing import Tuple

from cal_tools.enums import JungfrauSettings


def _get_settings(run_dc, ctrl_src) -> str:
    try:
        return(JungfrauSettings(
            run_dc.get_run_value(ctrl_src, "settings.value")))
    except KeyError:
        print(
            "WARNING: \'settings.value\' key is not available at "
            f"{run_dc.select(ctrl_src).files[0].filename},\n")
        return


class JungfrauCtrl():
    def __init__(
        self,
        run_dc: "extra_data.DataCollection",  # noqa
        ctrl_src: str,
    ):

        """Read slow data from RUN source.
        :param run_dir: EXtra-data RunDirectory DataCollection object.
        :param ctrl_src: Control source name for accessing run slow data.
        """
        self.run_dc = run_dc
        self.ctrl_src = ctrl_src
        self.run_settings = _get_settings(run_dc, ctrl_src)

    def get_memory_cells(self) -> Tuple[int, int]:
        n_storage_cells = int(self.run_dc.get_run_value(
            self.ctrl_src, "storageCells.value")) + 1
        storage_cell_st = int(self.run_dc.get_run_value(
            self.ctrl_src, "storageCellStart.value"))
        return n_storage_cells, storage_cell_st

    def get_bias_voltage(self) -> int:
        return(int(self.run_dc.get_run_value(
            self.ctrl_src, "vHighVoltage.value")[0]))

    def get_integration_time(self) -> float:
        return(float(self.run_dc.get_run_value(
            self.ctrl_src, "exposureTime.value")) * 1e6)

    def get_gain_setting(self) -> int:
        if self.run_settings == JungfrauSettings.DYNAMIC_GAIN_HG0:
            gain_setting = 1
        else:  # JungfrauSettings.DYNAMIC_GAIN
            gain_setting = 0
            if self.run_settings != JungfrauSettings.DYNAMIC_GAIN:
                print(
                    "WARNING: Setting gain_setting to 0, "
                    "assuming that this is an old run.\n")
        return gain_setting

    def get_gain_mode(self) -> int:
        if self.run_settings in [JungfrauSettings.FIX_GAIN_1, JungfrauSettings.FIX_GAIN_2]:  # noqa
            return 1
        else:
            return 0
